@@ -1,6 +1,11 @@
-from jobqueues.celeryfiles.celery import app
 from celery.exceptions import SoftTimeLimitExceeded
 import psutil
+from celery import task
+from billiard import current_process
+from jobqueues.util import _getVisibleGPUdevices
+
+
+visibledevs = _getVisibleGPUdevices()
 
 
 def kill(proc_pid):
@@ -10,8 +15,46 @@ def kill(proc_pid):
     process.kill()
 
 
-@app.task
-def run_simulation(folder, deviceid, sentinel, datadir, copyextensions, jobname=None):
+@task
+def execute_gpu_job(folder, sentinel, datadir, copyextensions, jobname=None):
+    import subprocess
+    import os
+    import time
+
+    worker_index = current_process().index
+    gpu_index = worker_index
+    if visibledevs is not None:
+        gpu_index = visibledevs[worker_index % len(visibledevs)]
+    print(f"Running job on worker index {worker_index} and GPU device {gpu_index}")
+
+    runsh = os.path.join(folder, "run.sh")
+    jobsh = os.path.join(folder, "job.sh")
+    stdfile = os.path.join(folder, "celery.out")
+    _createJobScript(
+        jobsh, folder, runsh, worker_index, sentinel, datadir, copyextensions
+    )
+
+    # Sleep for a short bit so that the OS can pick up on the new file before executing it
+    time.sleep(0.2)
+
+    process = None
+    try:
+        with open(stdfile, "a") as fout:
+            process = subprocess.Popen(
+                ["/bin/sh", jobsh], stdout=fout, stderr=fout, shell=False
+            )
+            _ = process.communicate()
+    except SoftTimeLimitExceeded:
+        # The SoftTimeLimitExceeded exception is a hack because SIGTERM doesn't work in Celery. See celeryqueue.py comment.
+        if process is not None:
+            kill(process.pid)
+        print(f"Job {jobname} has been cancelled by the user")
+    except Exception as e:
+        raise e
+
+
+@task
+def execute_cpu_job(folder, sentinel, datadir, copyextensions, jobname=None):
     import subprocess
     import os
     import time
@@ -19,7 +62,7 @@ def run_simulation(folder, deviceid, sentinel, datadir, copyextensions, jobname=
     runsh = os.path.join(folder, "run.sh")
     jobsh = os.path.join(folder, "job.sh")
     stdfile = os.path.join(folder, "celery.out")
-    _createJobScript(jobsh, folder, runsh, deviceid, sentinel, datadir, copyextensions)
+    _createJobScript(jobsh, folder, runsh, None, sentinel, datadir, copyextensions)
 
     # Sleep for a short bit so that the OS can pick up on the new file before executing it
     time.sleep(0.2)
